@@ -25,7 +25,7 @@ NULL
 
 #' @rdname load.model
 #' @export
-load_enm <- function(name, path = getwd()) {
+load_esdm <- function(name, path = getwd()) {
   path <- paste0(path, "/", name)
   a <- try(read.csv(paste0(path, "/Tables/AlgoCorr.csv"), row.names = 1))
   if (inherits(a, "try-error")) {
@@ -35,22 +35,22 @@ load_enm <- function(name, path = getwd()) {
   b <- try(raster(paste0(path, "/Rasters/Binary.tif")))
   if (inherits(b, "try-error")) {
     projection <- raster(paste0(path, "/Rasters/Probability.tif"))
-    evaluation <- read.csv(paste0(path, "/Tables/ENMeval.csv"), row.names = 1)
+    evaluation <- read.csv(paste0(path, "/Tables/ESDMeval.csv"), row.names = 1)
     b <- reclassify(projection, c(-Inf, evaluation$threshold, 0, evaluation$threshold,
                                   Inf, 1))
   }
-  enm <- Ensemble.SDM(name = as.character(read.csv(paste0(path, "/Tables/Name.csv"))[1,2]),
+  esdm <- Ensemble.SDM(name = as.character(read.csv(paste0(path, "/Tables/Name.csv"))[1,2]),
                       projection = raster(paste0(path, "/Rasters/Probability.tif")),
                       binary = b,
                       uncertainty = try(raster(paste0(path, "/Rasters/uncertainty.tif"))),
-                      evaluation = read.csv(paste0(path, "/Tables/ENMeval.csv"), row.names = 1),
+                      evaluation = read.csv(paste0(path, "/Tables/ESDMeval.csv"), row.names = 1),
                       algorithm.evaluation = read.csv(paste0(path, "/Tables/AlgoEval.csv"),
                                                       row.names = 1),
                       algorithm.correlation = a,
                       data = read.csv(paste0(path, "/Tables/Data.csv"), row.names = 1),
                       variable.importance = read.csv(paste0(path, "/Tables/VarImp.csv"), row.names = 1),
                       parameters = read.csv(paste0(path, "/Tables/Parameters.csv"), row.names = 1, colClasses = "character"))
-  return(enm)
+  return(esdm)
 }
 
 #' @rdname load.model
@@ -70,18 +70,18 @@ load_stack <- function(name = "Stack", path = getwd(), GUI = FALSE) {
                                              row.names = 1), variable.importance = read.csv(paste0(path,
                                                                                                    "/Stack/Tables/VarImp.csv"), row.names = 1), algorithm.correlation = a,
                        algorithm.evaluation = read.csv(paste0(path, "/Stack/Tables/AlgoEval.csv"),
-                                                       row.names = 1), enms = list(), parameters = read.csv(paste0(path,
+                                                       row.names = 1), esdms = list(), parameters = read.csv(paste0(path,
                                                                                                                    "/Stack/Tables/Parameters.csv"), row.names = 1, colClasses = "character"))
-  enms <- list.dirs(paste0(path, "/Species"), recursive = FALSE, full.names = FALSE)
+  esdms <- list.dirs(paste0(path, "/Species"), recursive = FALSE, full.names = FALSE)
   if (GUI) {
-    incProgress(1/(length(enms) + 1), detail = "stack main results")
+    incProgress(1/(length(esdms) + 1), detail = "stack main results")
   }
-  for (i in seq_len(length(enms))) {
-    cat(enms[i])
-    enm <- load_enm(enms[i], path = paste0(path, "/Species"))
-    stack@enms[[enm@name]] <- enm
+  for (i in seq_len(length(esdms))) {
+    cat(esdms[i])
+    esdm <- load_esdm(esdms[i], path = paste0(path, "/Species"))
+    stack@esdms[[esdm@name]] <- esdm
     if (GUI) {
-      incProgress(1/(length(enms) + 1), detail = enm@name)
+      incProgress(1/(length(esdms) + 1), detail = esdm@name)
     }
   }
   return(stack)
